@@ -8,12 +8,14 @@ LogRatioFGLasso <- function(x,
                             wcov,
                             a=1,
                             mu=1,
+                            maxiter=100,
                             ncv=5,
                             foldid=NULL,
                             step2=FALSE,
                             progress=TRUE,
                             plot=TRUE,
-                            mcv="Deviance"){
+                            mcv="Deviance",
+                            ncore=1){
   
   ptm <- proc.time()
   
@@ -58,7 +60,7 @@ LogRatioFGLasso <- function(x,
   
   if (progress) cat("Algorithm running for full dataset: \n")
   
-  fullfit <- fg_enet_al(x,t0,t1,d,tj,weight,length.lambda,mu,100,lambda,wcov,a,adjust,ncov,devnull,progress)
+  fullfit <- fg_enet_al(x,t0,t1,d,tj,weight,length.lambda,mu,maxiter,lambda,wcov,a,adjust,ncov,devnull,progress)
   lidx <- which(fullfit$loglik != 0 | !is.nan(fullfit$loglik))
   
   dev <- -2*fullfit$loglik[lidx]
@@ -78,70 +80,131 @@ LogRatioFGLasso <- function(x,
     if (is.null(foldid)){
       labels <- coxsplitss(as.matrix(y),id,ncv)
     }else{
-      labels <- foldid
+      if (length(foldid)==length(unique(id))){
+        labels <- fgfoldid(id,foldid)
+      }else{
+        labels <- foldid
+      }
     }
     
     # labels <- caret::createFolds(factor(d),k=ncv)
     
-    for (cv in 1:ncv){
+    if (ncore == 1){
       
-      if (progress) cat(paste0("Algorithm running for cv dataset ",cv," out of ",ncv,": \n"))
-      
-      # train.x <- x[-labels[[cv]],]
-      # train.d <- d[-labels[[cv]]]
-      # train.t <- t[-labels[[cv]]]
-      # test.x <- x[labels[[cv]],]
-      # test.d <- d[labels[[cv]]]
-      # test.t <- t[labels[[cv]]]
-      
-      train.x <- x[labels!=cv,]
-      train.d <- d[labels!=cv]
-      train.t0 <- t0[labels!=cv]
-      train.t1 <- t1[labels!=cv]
-      train.tt <- y[labels!=cv,1:2]
-      train.w <- weight[labels!=cv]
-      
-      test.x <- x[labels==cv,]
-      test.d <- d[labels==cv]
-      test.t0 <- t0[labels==cv]
-      test.t1 <- t1[labels==cv]
-      test.tt <- y[labels==cv,1:2]
-      test.w <- weight[labels==cv]
-      
-      cv.devnull <- 0
-      train.tj <- sort(train.t1[train.d==1])
-      for (j in 1:length(train.tj)){
-        cv.devnull <- cv.devnull + log(sum(train.t1 >= train.tj[1] & train.t0 < train.tj[1]))
+      for (cv in 1:ncv){
+        
+        if (progress) cat(paste0("Algorithm running for cv dataset ",cv," out of ",ncv,": \n"))
+        
+        # train.x <- x[-labels[[cv]],]
+        # train.d <- d[-labels[[cv]]]
+        # train.t <- t[-labels[[cv]]]
+        # test.x <- x[labels[[cv]],]
+        # test.d <- d[labels[[cv]]]
+        # test.t <- t[labels[[cv]]]
+        
+        train.x <- x[labels!=cv,]
+        train.d <- d[labels!=cv]
+        train.t0 <- t0[labels!=cv]
+        train.t1 <- t1[labels!=cv]
+        train.tt <- y[labels!=cv,1:2]
+        train.w <- weight[labels!=cv]
+        
+        test.x <- x[labels==cv,]
+        test.d <- d[labels==cv]
+        test.t0 <- t0[labels==cv]
+        test.t1 <- t1[labels==cv]
+        test.tt <- y[labels==cv,1:2]
+        test.w <- weight[labels==cv]
+        
+        cv.devnull <- 0
+        train.tj <- sort(train.t1[train.d==1])
+        for (j in 1:length(train.tj)){
+          cv.devnull <- cv.devnull + log(sum(train.t1 >= train.tj[1] & train.t0 < train.tj[1]))
+        }
+        cv.devnull <- 2*cv.devnull
+        
+        cvfit <- fg_enet_al(train.x,train.t0,train.t1,train.d,train.tj,train.w,length(lidx),mu,maxiter,lambda[lidx],wcov,a,adjust,ncov,cv.devnull,progress)
+        
+        cv.devnull <- 0
+        loglik <- rep(0,length(lidx))
+        linprod <- test.w * (test.x %*% cvfit$beta)
+        
+        if (mcv == "Deviance"){
+          
+          test.tj <- sort(test.t1[test.d==1])
+          for (j in 1:length(test.tj)){
+            cv.devnull <- cv.devnull + log(sum(test.t1 >= test.tj[j] & test.t0 < test.tj[j]))
+            if (sum(test.t1 >= test.tj[j] & test.t0 < test.tj[j]) > 1){
+              cvdev[,cv] <- cvdev[,cv] + linprod[test.t1 == test.tj[j],] - 
+                log(colSums(exp(linprod[test.t1 >= test.tj[j] & test.t0 < test.tj[j],])) + 1e-8)
+            }else if (sum(test.t1 >= test.tj[j] & test.t0 < test.tj[j]) == 1){
+              cvdev[,cv] <- cvdev[,cv] + linprod[test.t1 == test.tj[j],] - 
+                log(exp(linprod[test.t1 >= test.tj[j] & test.t0 < test.tj[j],]) + 1e-8) 
+            }
+            #- log(accu(link(widx))+1e-8)
+          }
+          
+          cvdevnull[cv] <- 2*cv.devnull
+          cvdev[,cv] <- -2*cvdev[,cv]
+          
+        }
+        
+        # cvmse[,cv] <- apply(cbind(1,test.x) %*% rbind(t(cvfit$beta0),cvfit$beta),2,function(x) sum((test.y - exp(x)/(1+exp(x)))^2)/length(test.y))
+        
       }
-      cv.devnull <- 2*cv.devnull
       
-      cvfit <- fg_enet_al(train.x,train.t0,train.t1,train.d,train.tj,train.w,length(lidx),mu,100,lambda[lidx],wcov,a,adjust,ncov,cv.devnull,progress)
+    }else if(ncore > 1){
       
-      cv.devnull <- 0
-      loglik <- rep(0,length(lidx))
-      linprod <- test.w * (test.x %*% cvfit$beta)
+      if (progress) warning(paste0("Using ", ncore ," core for cross-validation computation."))
       
-      if (mcv == "Deviance"){
+      cl <- makeCluster(ncore)
+      registerDoParallel(cl)
+      
+      cvdev <- foreach(cv=1:ncv,.combine=cbind) %dopar% {
+        
+        train.x <- x[labels!=cv,]
+        train.d <- d[labels!=cv]
+        train.t0 <- t0[labels!=cv]
+        train.t1 <- t1[labels!=cv]
+        train.tt <- y[labels!=cv,1:2]
+        train.w <- weight[labels!=cv]
+        
+        test.x <- x[labels==cv,]
+        test.d <- d[labels==cv]
+        test.t0 <- t0[labels==cv]
+        test.t1 <- t1[labels==cv]
+        test.tt <- y[labels==cv,1:2]
+        test.w <- weight[labels==cv]
+        
+        cv.devnull <- 0
+        train.tj <- sort(train.t1[train.d==1])
+        for (j in 1:length(train.tj)){
+          cv.devnull <- cv.devnull + log(sum(train.t1 >= train.tj[1] & train.t0 < train.tj[1]))
+        }
+        cv.devnull <- 2*cv.devnull
+        
+        cvfit <- fg_enet_al(train.x,train.t0,train.t1,train.d,train.tj,train.w,length(lidx),mu,maxiter,lambda[lidx],wcov,a,adjust,ncov,cv.devnull,FALSE)
+        
+        cv.dev <- rep(0,length(lidx))
+        linprod <- test.w * (test.x %*% cvfit$beta)
         
         test.tj <- sort(test.t1[test.d==1])
         for (j in 1:length(test.tj)){
-          cv.devnull <- cv.devnull + log(sum(test.t1 >= test.tj[j] & test.t0 < test.tj[j]))
           if (sum(test.t1 >= test.tj[j] & test.t0 < test.tj[j]) > 1){
-            cvdev[,cv] <- cvdev[,cv] + linprod[test.t1 == test.tj[j],] - 
+            cv.dev <- cv.dev + linprod[test.t1 == test.tj[j],] - 
               log(colSums(exp(linprod[test.t1 >= test.tj[j] & test.t0 < test.tj[j],])) + 1e-8)
           }else if (sum(test.t1 >= test.tj[j] & test.t0 < test.tj[j]) == 1){
-            cvdev[,cv] <- cvdev[,cv] + linprod[test.t1 == test.tj[j],] - 
+            cv.dev <- cv.dev + linprod[test.t1 == test.tj[j],] - 
               log(exp(linprod[test.t1 >= test.tj[j] & test.t0 < test.tj[j],]) + 1e-8) 
           }
-          #- log(accu(link(widx))+1e-8)
         }
         
-        cvdevnull[cv] <- 2*cv.devnull
-        cvdev[,cv] <- -2*cvdev[,cv]
+        cv.dev <- -2*cv.dev
+        cv.dev
         
       }
       
-      # cvmse[,cv] <- apply(cbind(1,test.x) %*% rbind(t(cvfit$beta0),cvfit$beta),2,function(x) sum((test.y - exp(x)/(1+exp(x)))^2)/length(test.y))
+      stopCluster(cl)
       
     }
     
@@ -164,6 +227,8 @@ LogRatioFGLasso <- function(x,
                 a=a,
                 loss=fullfit$loss[lidx],
                 mse=fullfit$mse[lidx],
+                tol=fullfit$tol[lidx],
+                iters=fullfit$iters[lidx],
                 cvdev.mean=mean.cvdev,
                 cvdev.se=se.cvdev,
                 best.beta=best.beta,
@@ -238,6 +303,8 @@ LogRatioFGLasso <- function(x,
             stepglmnet <- suppressWarnings(cv.glmnet(x=x.select.min,y=Surv(t0,t1,d),weights=weight,type.measure = "deviance",family="cox"))
             x.select.min <- x.select.min[,which(stepglmnet$glmnet.fit$beta[,stepglmnet$index[1]]!=0)]
             idxs <- idxs[,which(stepglmnet$glmnet.fit$beta[,stepglmnet$index[1]]!=0)]
+          }else{
+            idxs <- as.vector(idxs)
           }
           
           df_step2 <- data.frame(t0=t0,t1=t1,d=d,x=x.select.min)
@@ -272,6 +339,8 @@ LogRatioFGLasso <- function(x,
             stepglmnet <- suppressWarnings(cv.glmnet(x=x.select.min,y=Surv(t0,t1,d),weights=weight,type.measure = "deviance",family="cox"))
             x.select.min <- x.select.min[,which(stepglmnet$glmnet.fit$beta[,stepglmnet$index[1]]!=0)]
             idxs <- idxs[,which(stepglmnet$glmnet.fit$beta[,stepglmnet$index[1]]!=0)]
+          }else{
+            idxs <- as.vector(idxs)
           }
           
           df_step2 <- data.frame(t0=t0,t1=t1,d=d,x=x.select.min)
