@@ -131,6 +131,8 @@ Rcpp::List linear_enet_al(arma::mat x, arma::vec y, int len, double mu, int ub,
   //arma::vec lambda = vec(len);
   arma::vec loss = vec(len);
   arma::vec mse = vec(len);
+  arma::vec tol = vec(len);   
+  arma::vec iters = vec(len);
   
   arma::mat xx = x.t()*x;
   arma::vec xy = x.t()*y;
@@ -226,6 +228,7 @@ Rcpp::List linear_enet_al(arma::mat x, arma::vec y, int len, double mu, int ub,
     loss(i) = lossnew;
     mse(i) = accu(difftemp % difftemp)/n;
     beta.col(i) = betatemp;
+    tol(i) = mean(abs(betatemp - betai));iters(i) = k;
     
   }
   
@@ -234,6 +237,7 @@ Rcpp::List linear_enet_al(arma::mat x, arma::vec y, int len, double mu, int ub,
   ret["lambda"] = lambda;
   ret["loss"] = loss;
   ret["mse"] = mse;
+  ret["tol"] = tol; ret["iters"] = iters;
   
   return ret;
   
@@ -252,6 +256,8 @@ Rcpp::List logistic_enet_al(arma::mat x, arma::vec y, int len, double mu, int ub
   beta.zeros(p,len);
   arma::vec loss = vec(len);
   arma::vec mse = vec(len);
+  arma::vec tol = vec(len);   
+  arma::vec iters = vec(len);
   
   Progress prog(len, display_progress);
   
@@ -383,12 +389,17 @@ Rcpp::List logistic_enet_al(arma::mat x, arma::vec y, int len, double mu, int ub
           lossnew = accu(hfun % difftemp % difftemp)/(2*n)+ a*l*accu(abs(betatemp.subvec(ncov,p-1))) + (1-a)*l*accu(pow(betatemp.subvec(ncov,p-1),2))/2 + mu*pow(accu(betatemp.subvec(ncov,p-1)) + alpha,2)/2;    
         }
       }
+      
+      // Rcpp::Rcout << "k1: " << k1<< endl;
+      
     }
     
     beta0(i) = mean(z0 - x*betatemp);
     loss(i) = lossnew;
     mse(i) = accu(difftemp % difftemp)/n;
     beta.col(i) = betatemp;
+    tol(i) = mean(abs(betatemp - betai));
+    iters(i) = k;
     
   }
   
@@ -398,6 +409,8 @@ Rcpp::List logistic_enet_al(arma::mat x, arma::vec y, int len, double mu, int ub
   ret["lambda"] = lambda;
   ret["loss"] = loss;
   ret["mse"] = mse;
+  ret["tol"] = tol; 
+  ret["iters"] = iters;
   
   return ret;
   
@@ -415,6 +428,9 @@ Rcpp::List cox_enet_al(arma::mat x, arma::vec t, arma::vec d, arma::vec tj, int
   beta.zeros(p,len);
   arma::vec loss = vec(len);
   arma::vec mse = vec(len);
+  arma::vec tol = vec(len);   
+  arma::vec iters = vec(len);
+  
   arma::vec loglik;
   loglik.zeros(len);
   
@@ -658,6 +674,8 @@ Rcpp::List cox_enet_al(arma::mat x, arma::vec t, arma::vec d, arma::vec tj, int
     loss(i) = lossnew;
     mse(i) = accu(difftemp % difftemp)/n;
     beta.col(i) = betatemp;
+    tol(i) = mean(abs(betatemp - betai));
+    iters(i) = k;
     
     if (i == len-1){
       
@@ -679,6 +697,8 @@ Rcpp::List cox_enet_al(arma::mat x, arma::vec t, arma::vec d, arma::vec tj, int
   ret["loss"] = loss;
   ret["mse"] = mse;
   ret["loglik"] = loglik;
+  ret["tol"] = tol; 
+  ret["iters"] = iters;
   
   return ret;
   
@@ -696,6 +716,8 @@ Rcpp::List cox_timedep_enet_al(arma::mat x, arma::vec t0, arma::vec t1, arma::ve
   beta.zeros(p,len);
   arma::vec loss = vec(len);
   arma::vec mse = vec(len);
+  arma::vec tol = vec(len);   
+  arma::vec iters = vec(len);
   arma::vec loglik;
   loglik.zeros(len);
   
@@ -849,6 +871,8 @@ Rcpp::List cox_timedep_enet_al(arma::mat x, arma::vec t0, arma::vec t1, arma::ve
     loss(i) = lossnew;
     mse(i) = accu(difftemp % difftemp)/n;
     beta.col(i) = betatemp;
+    tol(i) = mean(abs(betatemp - betai));
+    iters(i) = k;
     
     if (i == len-1){
       
@@ -870,6 +894,8 @@ Rcpp::List cox_timedep_enet_al(arma::mat x, arma::vec t0, arma::vec t1, arma::ve
   ret["loss"] = loss;
   ret["mse"] = mse;
   ret["loglik"] = loglik;
+  ret["tol"] = tol; 
+  ret["iters"] = iters;
   
   return ret;
   
@@ -887,6 +913,8 @@ Rcpp::List fg_enet_al(arma::mat x, arma::vec t0, arma::vec t1, arma::vec d, arma
   beta.zeros(p,len);
   arma::vec loss = vec(len);
   arma::vec mse = vec(len);
+  arma::vec tol = vec(len);   
+  arma::vec iters = vec(len);
   arma::vec loglik;
   loglik.zeros(len);
   
@@ -1039,6 +1067,8 @@ Rcpp::List fg_enet_al(arma::mat x, arma::vec t0, arma::vec t1, arma::vec d, arma
     loss(i) = lossnew;
     mse(i) = accu(difftemp % difftemp)/n;
     beta.col(i) = betatemp;
+    tol(i) = mean(abs(betatemp - betai));
+    iters(i) = k;
     
     if (i == len-1){
       
@@ -1060,7 +1090,9 @@ Rcpp::List fg_enet_al(arma::mat x, arma::vec t0, arma::vec t1, arma::vec d, arma
   ret["loss"] = loss;
   ret["mse"] = mse;
   ret["loglik"] = loglik;
-  
+  ret["tol"] = tol; 
+  ret["iters"] = iters;
+   
   return ret;
   
 }
